#include <linux/module.h>
#include <linux/modversions.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <asm/byteorder.h>
#include <linux/crypto.h>

/*
 * Simple cipher_impl wrapper around openssl 686 asm crypto
 *
 * Copyright (C) 2000 Zero-Knowledge[R] Systems, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "openssl/blowfish.h"

#define KEY_SIZE 16

struct blowfish_context {
        BF_KEY key_schedule;
        unsigned char iv[BF_BLOCK];
};

int blowfish_encrypt(struct cipher_context *cx, 
                             const u8 *in8, u8 *out8, int size) 
{
    struct blowfish_context *bfctx = (struct blowfish_context *)cx->keyinfo;

    BF_cbc_encrypt( in8, out8, size,
                    &(bfctx->key_schedule), bfctx->iv, BF_ENCRYPT);
    return 0;
}

int blowfish_decrypt(struct cipher_context *cx, 
                             const u8 *in8, u8 *out8, int size) 
{
    struct blowfish_context *bfctx = (struct blowfish_context *)cx->keyinfo;

    BF_cbc_encrypt( in8, out8, size,
                    &(bfctx->key_schedule), bfctx->iv, BF_DECRYPT);

    return 0;
}

int blowfish_set_key(struct cipher_context *cx, 
                             unsigned char *key, int keybytes) 
{
    struct blowfish_context *bfctx = (struct blowfish_context *)cx->keyinfo;

    /* XXX put in explicit check for keylen */

    BF_set_key(&(bfctx->key_schedule), keybytes, key);
    memset(bfctx->iv, 0, sizeof(bfctx->iv));

    return 0;

}

static void blowfish_lock(void) 
{
    MOD_INC_USE_COUNT;
}

static void blowfish_unlock(void) 
{
    MOD_DEC_USE_COUNT;
}

#define CIPHER_BITS_64
#define CIPHER_NAME(x) blowfish##x
DEFINE_CIPHER(BLOWFISH,     blowfish, "blowfish", 8, 8);

int __init init_module(void)
{
    if (register_cipher(&blowfish)) {
        printk(KERN_WARNING "Couldn't register blowfish encryption\n");
    } else {
        printk("OpenSSL blowfish crypto routines registered.\n");
        printk("  \"This product includes cryptographic software\n"
                            "   written by Eric Young (eay@cryptsoft.com)\"\n");
    }
    return 0;
}

void cleanup_module(void)
{
    if (unregister_cipher(&blowfish))
        printk(KERN_WARNING "Couldn't unregister blowfish encryption\n");
}
